/*
 * Copyright (c) 2008 Motorola, Inc. or its subsidiaries. 
 * All rights reserved.
 */


/***********************************************/
/****************** SCROLL BAR  ****************/
/***********************************************/


/**
 * Scroll Bar
 * 
 * Enable developers to create customized Scroll Bars that they can control.
 * 
 * There are two kinds/modes of Scroll Bars:
 * 
 * 	- "list" mode:
 * 	  This mode must be used when the Scroll Bar will be used to represent a list 
 *    of items (see Navigation List component, for example). In this mode the Scroll 
 *    Bar will represent a certain amount of objects but it will not handle anything, 
 *    the developer needs to call the methods to scroll up/down.
 * 	
 * 	- "page" mode:
 *    This mode must be used when the Scroll Bar will be used to handle some
 * 	  page navigation (in this case, the parameters containerDivId and contentDivId
 *    must be provided)
 * 
 *
 * # Available methods: 
 * 
 * 		restart
 * 		refresh
 * 		keyDown
 * 		scroll
 * 
 *  
 * # Usage examples:
 * 
 * 		var scrollBar = new ScrollBar(<mode>, <sbId> , <sId>, <containerDivId>, <contentDivId>)
 * 
 * 		// refresh scroll size
 * 		scrollBar.refresh();
 * 
 * 		// restart navigation
 * 		scrollBar.restart();
 * 
 * 		// key down handling
 * 		scrollBar.keyDown(keyCode);
 * 
 * 		// scroll
 * 		scrollBar.scroll("down");
 *  
 * 
 * @param mode					[REQUIRED] {"list", "page"}		
 * @param sbId					[REQUIRED] scroll bar DIV ID
 * @param sId					[REQUIRED] scroll DIV ID (MUST be a child of sbId)
 * @param containerDivId		[mode="page" REQUIRED] external DIV ID (MUST have a fixed height and the CSS property 'overflow' set to 'hidden')
 * @param contentDivId			[mode="page" REQUIRED] content DIV ID (MUST be a child of containerDivId and NOT have a fixed height - its height must follow its content)
 *
*/
function ScrollBar(mode, sbId, sId, containerDivId, contentDivId){
	
	/*
	 * constants
	 */ 
	this.LIST_MODE = "list";				// Scroll Bar mode
	this.PAGE_MODE = "page";				// Scroll Bar mode
	
	/*
	 * attributes
	 */ 
	this.mode = mode;
	this.scrollBarId = sbId;
	this.scrollId = sId;
	this.containerDiv = containerDivId;
	this.contentDiv = contentDivId;	
	
	/*
	 * vars
	 */
	var position = 0;
	var scrollStepPage = 0.333333;				// percentage of the page that will be scrolled up/down (page mode)
	var scrollStepList = undefined;			// scroll step for the list mode 

	/*
	 * contants 
	 */
	var KEY_CODE_UP = 38;					// navigation pad map
	var KEY_CODE_DOWN = 40;					// navigation pad map
	
	
	/**
	* Restart navigation
	*
	* Position the scroll at the top of the Scroll Bar as well as go to the
	* top of the page if the "page" mode is set.
	*/
	this.restart = function(){
		
		// the scroll is at the initial position 
		position = 0;
		
		switch(this.mode){
		
			case this.PAGE_MODE:
				document.getElementById(this.scrollId).style.marginTop = "0px" ;
				document.getElementById(this.contentDiv).style.marginTop = "0px";
				break;
				
			case this.LIST_MODE:
				document.getElementById(this.scrollId).style.marginTop = "0px" ;
				break;
		}
		
	}
	
	/**
	* Refresh the Scroll Bar, adjusting the scroll size
	*
	* If the mode is "page", it will sdjust the scroll bar according to the contentDiv height
	* if the mode is "list", it will sdjust the scroll bar according to the number of items passed as a parameter
	* 
	* @param  numItems		[mode="list" REQUIRED] number of 'items' the Scroll Bar will represent
	*/
	this.refresh = function (numItems){
		
		switch(this.mode){
		
			case this.PAGE_MODE:
				
				var visiblePercentage =  (document.getElementById(this.containerDiv).clientHeight) / (document.getElementById(this.contentDiv).clientHeight);
		
				// all the page is displayed
				if (visiblePercentage >= 1) {
					document.getElementById(this.scrollId).style.display = "none";
					document.getElementById(this.scrollId).style.height = "0px" ;
				} else {  // just part of the page is displayed
					var scrollHeight = (document.getElementById(this.scrollBarId).clientHeight) * visiblePercentage;
					document.getElementById(this.scrollId).style.display = "block";
					document.getElementById(this.scrollId).style.height = scrollHeight +  "px";
					// adjust the scroll position
					document.getElementById(this.scrollId).style.marginTop = (Number(position) * Number(scrollStepPage) * Number(scrollHeight)) + "px";
				}
				
				break;
			
			case this.LIST_MODE:
				
				// 0 or 1 item will not activate the scroll bar
				if (numItems <= 1){
					document.getElementById(this.scrollId).style.display = "none";
					scrollStepList = 0;
				} else {  // more than 1 item will active the scroll bar
					scrollStepList = Math.round(Number((document.getElementById(this.scrollBarId).clientHeight) / numItems)) ;
					document.getElementById(this.scrollId).style.display = "block";
					document.getElementById(this.scrollId).style.height = scrollStepList + "px";
					// adjust the scroll position
					document.getElementById(this.scrollId).style.marginTop = (Number(position) * Number(scrollStepList)) + "px";
				}
				
				break;
				
			}
		
	}
	
	
	/**
	* Key down handling
	*
	* Move scroll up or down if the "up" or "down" key has been pressed 
	*
	* @param  keyCode	key pressed by the user
	*/
	this.keyDown = function(keyCode){
		
		switch(keyCode){
			
			case KEY_CODE_UP:
				this.scroll("up");
				break;
				
			case KEY_CODE_DOWN:
				this.scroll("down");
				break;
		
		}
		
	}
	
	
	/**
	* Move scroll up or down
	* 
	* @param  direction		{"up", "down"}
	* @param  steps			how many times the scroll has to be moved
	*/
	this.scroll = function(direction, steps){
		
		if (steps == undefined) steps = 1;
		steps = Number(steps);
		
		var marginTop = document.getElementById(this.scrollId).style.marginTop;
		var marginTopPixels = (((marginTop != "") && (marginTop != undefined)) ? Number(marginTop.substring(0, (marginTop.length - 2))) : 0);
		var scrollHeight = document.getElementById(this.scrollId).style.height;
		var scrollHeightPixels = (((scrollHeight != "") && (scrollHeight != undefined)) ? Number(scrollHeight.substring(0, (scrollHeight.length - 2))) : 0);
		
		switch(direction){
			case "up":
				if(position > 0) {
					if(this.mode == this.PAGE_MODE){
						if (document.getElementById(this.scrollId).style.display != "none"){
							if(marginTopPixels > 1){
								// scroll
								if (Number(marginTopPixels - (scrollHeightPixels * steps * scrollStepPage)) < 0){
									while(Number(marginTopPixels - (scrollHeightPixels * steps * scrollStepPage)) < 0){
										steps--;
									}
									steps++;
								}
								document.getElementById(this.scrollId).style.marginTop = Number(marginTopPixels - (scrollHeightPixels * steps * scrollStepPage)) + "px";
								
								// page
								var marginTopPage = document.getElementById(this.contentDiv).style.marginTop;
								var marginTopPagePixels = (((marginTopPage != "") && (marginTopPage != undefined)) ? Number(marginTopPage.substring(0, (marginTopPage.length - 2))) : 0);
								document.getElementById(this.contentDiv).style.marginTop = marginTopPagePixels + (document.getElementById(this.containerDiv).clientHeight * steps * scrollStepPage) + "px";
							}
						}
					} else {
						document.getElementById(this.scrollId).style.marginTop = Number(marginTopPixels - (steps * scrollStepList)) + "px"  ;
					}
					position = position - steps;			
				}	
				break;
				
			case "down":
				var hasScroll = 0;
				if(this.mode == this.PAGE_MODE){
					if (document.getElementById(this.scrollId).style.display != "none"){
						if (Number((marginTopPixels + scrollHeightPixels)) < Number(document.getElementById(this.scrollBarId).clientHeight)){
							// scroll
							if(Number(scrollHeightPixels + marginTopPixels + (scrollHeightPixels * steps * scrollStepPage)) > Number(document.getElementById(this.scrollBarId).clientHeight)){
								while(Number(scrollHeightPixels + marginTopPixels + (scrollHeightPixels * steps * scrollStepPage)) > Number(document.getElementById(this.scrollBarId).clientHeight)){
									steps--;
								}
								steps++;	
							}

							document.getElementById(this.scrollId).style.marginTop = Number(marginTopPixels + (scrollHeightPixels * steps * scrollStepPage)) + "px" ;

							// page
							var marginTopPage = document.getElementById(this.contentDiv).style.marginTop;
							var marginTopPagePixels = (((marginTopPage != "") && (marginTopPage != undefined)) ? Number(marginTopPage.substring(0, (marginTopPage.length - 2))) : 0);
							document.getElementById(this.contentDiv).style.marginTop = Number(marginTopPagePixels - (document.getElementById(this.containerDiv).clientHeight * steps * scrollStepPage)) + "px";
							hasScroll = 1;
						}
					}
				} else {
					document.getElementById(this.scrollId).style.marginTop = Number(marginTopPixels + (steps * scrollStepList)) + "px"  ;
				}
				if(hasScroll > 0) {
					position = position + steps;
				}
				break;
		}	 
	}
}